
c		****************************************************************
c
c			CFDYNAMICS : Coupled Free/porous Flow Simulation Program
c
c						Developed by  : Atul Ninu Waghode
c
c				Supervised by : Prof V.Nassehi & Prof. R.J.Wakeman
c    
c						Latest Version: 15th August, 2005
c
c		****************************************************************
c									-----------
c									Description
c									-----------
c
c							Program Title: ! StokesDarcyUVP.for ! 
c
c
c
c
c
c
c
c   ------------------------
c   TECHNICAL SPECIFICATIONS
c   ------------------------
c   
c   1. The program has been developed using a windows based FORTRAN 90 compiler;
c     
c   2. This program stores the basic data required for simulating coupled free/porous 
c      hydrodynamics in pleated cartridge filters; 
c
c   3. This program prompts the user to input the data in form of data file "datafile.txt";
c
c   4. This is a program for the solution of generalised Newtonian, isothermal,incompressible  
c      fluid flow problems in free flow channel confined by porus media using the weighted  
c      residual galerkin finite element method;
c
c   6. The solution scheme is based on the U-V-P method; 
c
c   7. Velocity components and pressure are the prime unknowns in the flow field;
c
c   8. Percentage compression and loss in are are also evaluted for a given
c      cartridge filter specification;
c
c  10. Simaltaneous algebraic equations are solved by a frontal method;
c
c  11. All the physical entities are measured and calculated in SI units;
c
c  11. The program consists of a main module and subroutines.
c
c
c   =================================================================================================================================================
c
c   work files
c   ==========
c     unit    |  contents
c   =============================================================
c       51    i    input data file
c             i
c       60    i    output file for documentation
c             i
c       11    i    output file containing velocity field data for
c             i    plotting
c             i
c       14    i    used as a work file in the solver routine
c             i
c       15    i    stores shape functions and their derivatives at
c             i    integration points
c             i
c       81    i    input file containing coordinates of 4-node element 
c             i    mesh for contour plotting of pressure
c             i
c       101   i    output file containing nodal values of primitive  
c             i    variables in 4-node element mesh
c             i
c ==================================================================
c
c     List of variables
c     =================
c     aa				    element coefficient matrices on LHS;
c     ak				    element coefficient matrices on RHS; 
c     b					global derivatives of shape functions;
c     bc					nodal constraints (boundary conditions);
c	cg					Weighting functions for Gauss integration;
c     cord				nodal coordinates;
c	da					measure of Gaussian integration;
c     del					local  derivatives of shape functions;
c	detj				Determinant of Jacobian matrix;
c	errp				Square of discrepancy between pressure values calculated 
c						in two successive iterations;
c	errv				Square of discrepancy between velocity values calculated 
c						in two successive iterations;
c	eta					y-coordinate in local coordinate system;
c     gamad               shear rate;
c     grav1               first  component of the applied body force;
c     grav2               second component of the applied body force;
c     icord               indicates whether the coordinate system is cartesian (planar) or
c                         cylindrical (axisymmetric);
c	maxbc				Limit for maximum number of boundary constraints 
c						that can be specified;
c	maxdf				Limit for maximum number of nodal degrees of freedom;
c	maxel				Limit for maximum number of elements in mesh;
c	maxnp				Limit for maximum number of nodal points in mesh;
c	maxst				Maximum dimension of finite element stiffness matrix;
c     nbc                 total number of boundary-node constraints;
c     ncn                 number of nodes per element;
c     ndf                 degree of freedom per node;
c     ndim                dimensions of the solution domain;
c     nel                 total number of elements;
c     ngaus               number of integration points;		
c     nnp                 total number of nodal points;
c     node				element connectivity;
c     nter                maximum number of iterations; 
c     num                 number of integration points per element;
c     p					shape functions;
c     power               power law index;
c     pres				nodal presures;
c     r1					global load vector on RHS;
c     rpres               reference presure;
c     rr					element load vector;
c     rtem                reference temperature;
c     rvisc               consistency coefficient in power-law model;
c	sinv				Array storing components of second invariant of rate of 
c						deformation of stress tensor;
c	soln				Array for storing solution values calculated by Frontal Solver;
c     spres               presure;
c     stemp               temperature;
c     tolc                convergence tolerance factor for concentration;
c     tolp                convergence tolerance factor for presures;
c     tolv                convergence tolerance factor for velocities;
c     vel					nodal velocities;  
c	visc				Updated value of viscosity according to power law model;
c	xg					Coordinates of Gauss quadrature points for integration;
c	xi					x-coordinate in local coordinate system.



c     =======================================================================
c     List of Subroutines
c     ===================
c
c     bacsub              backsubstitution method for finding the final
c                         solution vector
c
c     clean               cleans the arrays and prepares them for
c                         solution
c
c     conc                calculates the concentrations
c
c     contol              makes a check for the convergence
c
c     deriv               calculates the jacobian matrix, its determinant
c                         and	global derivatives of the shape functions
c
c     flow                calculates the velocities and presures
c
c     front               frontal method for solving the final set
c                         of equations
c
c     gaussp              specifies the gauss points and weights for
c                         quadrature integration
c
c     getbcd              reads the primary boundary conditions
c
c     getelm              reads the nodal connectivity data
c
c     getmat              reads the input material data
c
c     getnod              reads the nodal co-ordinates
c
c     output              prints the final solution
c
c     putbcv              imposes the primary boundary conditions for
c                         velocities and presure
c
c     secinv              calculates the rate of deformation of stress
c                         tensor
c
c     setprm              sets the location data for nodal degrees of
c                         freedom (velocities & presures) 
c
c     shape               calculates the shape functions and their
c                         derivatives
c
c     visca               calculates the viscosity 
c
c    ================================================================
c
      parameter (maxel  = 6000    )
	parameter (maxnp  = 25000    )
	parameter (maxbc  = 1500     )
      parameter (maxdf  = maxnp*3 )
      parameter (maxst  = 27      ) 
	parameter (maxfr  = 1000    )
      parameter (ndim   = 2       )
	parameter (num    = 9       )

       implicit double precision(a-h,o-z)
c
c    Storage allocation
c    ==================
c      
	dimension title (         80)
      dimension node  (maxel,   27) ,pmat (maxel,  9) ,cord (maxnp,  2)
      dimension ncod  (maxdf      ) ,bc   (maxdf    ) ,gord (maxnp,  2)
      dimension ibc   (maxnp      ) ,jbc  (maxnp    ) ,vbc  (maxnp    )
      dimension vel   (maxnp,ndim ) ,pres (maxnp    ) ,qres (maxnp	)
	dimension r1    (maxdf      ) ,sinv (maxel,  9) ,rr   (maxst    ) 
      dimension vet   (maxdf      ) ,rel(maxnp,ndim ) ,b    (2,      9)
	dimension nopp  (maxdf      ) ,khed (maxfr    ) ,jmod (maxfr    )
      dimension aa    (maxst,maxst) ,nk   (maxst    )
      dimension xg    (          3) ,cg   (        3)
      dimension p     (          9) ,del  (    2,  9) 
      dimension ldest (maxst      ) ,kdest(maxst    ) 
      dimension eq    (maxfr,maxfr) ,lhed (maxfr    ) 
      dimension kpiv  (maxfr      ) ,lpiv (maxfr    ) 
      dimension qq    (maxfr      ) ,pvkol(maxfr    )
      dimension mdf   (maxdf      ) ,ndn  (maxdf    ) 
	dimension pp    (          4) ,delp (    2,  4)
      dimension soln  (maxdf      ) ,roln (maxdf	)	
	
	integer   rode  (maxel,   27),qnp,qel
	integer iii1,iii2,iii3,iii4,jjj1,jjj2,jjj3,jjj4,bbb1,ccc1
	integer bbb2,ccc2,bbb3,ccc3,bbb4,ccc4

      character *20 filname

c    ====================================================================================================================================

c	=====================================
c    Opening of input and output data files 
c    ======================================
c      
c     This data file contains the Numerical Parameters and Nodal Connectivity
c     -----------------------------------------------------------------------

	print *, "      
	1* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *"
	print *, "
	1     *                                                           *"
	print *, "
	1     *       ######  ######  #     # ####### #       #######     *"
	print *, "
	1     *      #       #      # #     # #     # #       #           *"
	print *, "
	1     *      #       #      # #     # #     # #       #           *"
	print *, "
	1     *      #       #      # #     # ####### #       ####        *"
	print *, "
	1     *      #       #      # #     # #       #       #           *"
	print *, "
	1     *      #       #      # #     # #       #       #           *"
	print *, "
	1     *       ######  ######   #####  #       ####### #######     *"
	print *, "
	1     *                                                           *"
	print *, "
	1     *                                                           *"
	print *, "
	1     *      CFDYNAMICS       August 2005                         *"
	print *, "
	1     *                                                           *"
	print *, "
	1      * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *"


	write(*, 1101)

	write(*, 1111)	

	write(*, 1121)

	write(*, 1131)

	print *, "
	1            Program Developer:  Atul Ninu Waghode"
	print *, "
	1  Project Advisors: Prof. R.J. Wakeman and Prof. V. Nassehi"
	print *, "
	1          Advanced Separation Technologies Group"
	print *, "
	1           Department of Chemical Engineering"
	print *, "
	1                Loughborough University"
	print *, "
	1             Loughborough, Leicestershire"
	print *, "
	1               LE11 3TU, United Kingdom"

	write(*, 1131)

	write (*, 1101)

c     ======================================================================



c     ======================================
c     Opening of input and output data files 
c     ======================================
c      
c
	print*,'Enter the name of DEFAULT data file'
	read(*,2000) filname	

	open(unit=51         ,   file=filname     ,   access='sequential',   
	1     form='formatted',   status="unknown" ,   iostat=ios         )

c     This is the output file containing the results
c     ------------------------------------------------------------------
	open(unit=60         ,   file='aerout.txt',   access='sequential',
	1     form='formatted',   status="unknown" ,   iostat=ios         )

	open(unit=101         ,   file='out-4.txt',   access='sequential',
	1     form='formatted',   status="unknown" ,   iostat=ios         )

c     These are scratch files used in computation
c     ------------------------------------------------------------------

      open(unit=14         ,   form='unformatted',  status='scratch'   ,
     1     iostat=ios                                                  )
      open(unit=15         ,   form='unformatted',  status='scratch'   ,
	1     iostat=ios                                                  )

c     These are auxilliary files used in computation
c     ------------------------------------------------------------------

     	open(unit=11         ,   file='interm3v'  ,   access='sequential',
	1     form='formatted',   status="unknown" ,   iostat=ios         )

     	open(unit=20         ,   file='stiffmat'  ,   access='sequential',
	1     form='formatted',   status="unknown" ,   iostat=ios         )

	open(unit=81         ,   file='CORD-4.FEM',   access='sequential',
	1     form='formatted',   status="unknown" ,   iostat=ios         )


c     These are formatted output files used for postprocessing in tecplot
c     -------------------------------------------------------------------

	open(unit=301         ,   file='tp.dat',   access='sequential',
	1     form='formatted',   status="unknown" ,   iostat=ios         )

	open(unit=401         ,   file='tp4.dat',   access='sequential',
	1     form='formatted',   status="unknown" ,   iostat=ios         )

c     ============================================================================================================================

c     Checks for opening of files
c     ---------------------------
      	
	  if(ios==0)then
			print*,"files opened    "
	  else 
			print*,"files not opened"
	  stop  
	  end if

	rewind 20	
	rewind 51
	rewind 60
	rewind 81
	rewind 101
	rewind 301
	rewind 401

c
c    ================= 
c    Initialize arrays
c    =================

      do 5010 itl = 1,maxel
      do 5010 ivl = 1,maxst
                          node (itl,ivl) = 0
						rode (itl,ivl) = 0
5010  continue
      do 5020 itl = 1,maxel
      do 5020 ivl = 1,ncn
                          pmat (itl,ivl) = 0.0
5020  continue
      do 5030 itl = 1,maxnp
      do 5030 ivl = 1,ndim
                          cord (itl,ivl) = 0.0
						vel  (itl,ivl) = 0.0 
5030  continue
      
      do 5031 itl = 1,maxel
      do 5031 ivl = 1,9
                          sinv (itl,ivl) = 0.0 
5031  continue

      do 5060 itl = 1,maxdf
                          nopp   (itl)   = 0
						mdf    (itl)   = 0
						ndn    (itl)   = 0
						ncod   (itl)   = 0
                          r1     (itl)   = 0.0
                          soln   (itl)   = 0.0
						roln   (itl)   = 0.0
						bc     (itl)   = 0.0
                          vet    (itl)   = 0.0
                                                   
5060  continue

      do 5080 itl = 1,maxnp
                          ibc   (itl)   = 0
                          jbc   (itl)   = 0
                          vbc   (itl)   = 0.0
						pres  (itl)   = 0.0
						qres  (itl)   = 0.0
5080  continue


c===================================================================================================


c     ====================
c     Title of the program                
c     ====================

	if   (.not. eof(51)) read (51,2010) title
      write(60,4010)       title
c
c     Element description data
c     ========================
     
      if  (.not. eof(51)) read (51,2020) ncn, ngaus, nter
      write(60,4020)      ncn ,ngaus

c
c     Mesh, boundary condition and material parameters
c     ================================================
	
	if (.not. eof(51)) read (51,2030) nnp, nel, nbc, nmat
	if (.not. eof(51)) read (51,2040) ntep ,icord
c
      if(icord.eq.0)     write(60,4030)
      if(icord.eq.1)     write(60,4040)

      write(60,4050)

	if (.not. eof(81)) read (81,2859) qnp, qel

	print *, "qnp = ", qnp
	print *, "qel = ", qel
     
      if(ntep.eq.0) ntep=1
c
c
c    ===========================================================
c    icord=0 coordinate system is cartesian   (   planar   )
c    icord=1 coordinate system is cylindrical (axisymmetric)
c
c    if ntep = 1 then computed result after every iteration will
c    be printed ;if you do not need the result of intermediate
c    computations choose your own ntep;the result of first and
c    converged solutions will always be printed.
c    ===========================================================
c
c
                if(nnp   .eq.0  .or.nnp  .gt.maxnp) then
                                             write(60,4060)
            elseif(nel   .eq.0  .or.nel  .gt.maxel) then
                                             write(60,4060)
            elseif(nbc   .eq.0  .or.nbc  .gt.maxbc) then
                                             write(60,4060)
            elseif(nmat  .eq.0  .or.nmat .gt.maxel) then
                                             write(60,4060)
            
		  print*, "the program is aborted"
            stop
	      
		      endif


	write(60,4070)     nnp ,nel ,nbc ,nmat
    
      if (.not. eof(51)) read(51,2060)  tolv    , tolp    

	if (.not. eof(51)) read(51,2070)  permx   , permy 
	 
      
c     ===========================================================================
c     Read input data from main data file and prepare arrays for solution process
c     ===========================================================================

c      Reads material properties
c      -------------------------------------------------
       call getmat (nel , nmat, pmat,  51,  60  ,  maxel,  rtem, rpef)  

c      Reads the nodal co-ordinates from input data file
c      -------------------------------------------------
	 call getnod (nnp ,qnp,cord,gord,51,60,81,91,maxnp,  ndim , icord)
       
c      Reads the Nodal Connectivity from the input data file
c      -------------------------------------------------
	 call getelm   (nel ,  ncn ,   node,  rode, 51,60, 81,91,maxel)
       
c      Reads the Nodal Constraints from the input data file
c      -------------------------------------------------
	 call getbcd   (nbc ,  ibc ,   jbc , vbc,  51   ,  60  , maxbc )
	

c    ==========================================================================
c						 Start of the iteration loop
c    ==========================================================================


c     Set control parameters (default values are overwritten by input data
c     if specified)
c
c     ncn      number of nodes per element
c     ngaus    number of integration points
c     nter     maximum number of iterations for non-newtonian case
c     ndim     number of space dimensions in the solution domain
c
      	                               do 5125 iel = 1,maxel
	                                   do 5125 lg  = 1,ncn
	                                   sinv(iel,lg)= 0.25
5125                                     continue
                                         do 5130 ivel = 1,maxnp
	                                   do 5130 ind  = 1,ndim
	                                   vel(ivel,ind)= 0.0
5130                                     continue
                                         

      

      do 5150 iter = 1 ,nter

		print *, "iter = ", iter
	
		write(60,4090) iter
		write(101,4090) iter

c    =================================================================================
c                         Calculate Nodal Velocities & presures
c    =================================================================================
      
	icho=1
                                                 rewind 11
                                                 rewind 14
                                                 rewind 15

                             ndf   = 3
                             ntov  = ndf * nnp
                             ntrix = ndf * ncn

      call clean
     1(ncn  ,  nel   ,  ndf  ,   node ,   rode, r1   ,   maxel,   maxst,
     2     maxdf,   bc    ,   ncod ,   icho                            )

      call setprm
     1    (nnp  ,   nel   ,   ncn  ,   node ,   ndf  ,  maxel,   maxst,
     2     ndn  ,   ntrix ,   maxdf,   mdf  ,  nopp          )

      call putbcv
     1    (nnp  ,   nbc  ,    ibc  ,   jbc  ,   vbc  ,  ncod,    bc   ,
     2     maxbc,   maxdf,    maxel,   node ,   nel    				 )


c     idv4 is the file specifier for unit=20
c     ======================================
 
      idv4=20			 

      call flow (node  ,   cord ,   pmat ,   nopp ,   mdf  ,   ndn     ,   
	1           ncod  ,   bc   ,   vel  ,   pres ,   r1   ,   ldest   ,
     2           kdest ,   nk   ,   eq   ,   lhed ,   khed ,
     3           kpiv  ,   lpiv ,   jmod ,   qq   ,   pvkol,   iter    ,
     4           nel   ,   ncn  ,   ngaus,   grav1,   permx,   permy   ,
     5           grav2 ,   p    ,   del  ,   b    ,   ntrix,   maxel   ,
     6           maxnp ,   maxst,   maxfr,   maxdf,   ndim ,   aa      ,
     7           xg    ,   da   ,   ntov ,   icord,   idv4 ,   sinv    ,
     8           nnp   ,   pp   ,   delp ,   soln ,   num              )                               

c    =================================================================================
c                         Calculate Nodal Velocities & presures
c    =================================================================================



      call secinv
     1         (nel  ,     nnp  ,   ncn  ,   ngaus,   node ,   sinv    ,
     2          cord ,     p    ,   b    ,   del  ,   da   ,   dis     , 
     3          maxnp,     maxel,   maxst,   ndim ,   icord,   maxdf   ,
	4          num                                                    )




c     Assiging the values of solution vectors to primitive variables
c     ===============================================================================

      do 5165		i11 =1,nnp
	            i22 =i11+nnp
				i33 =i22+nnp
	
	vel(i11,1)=soln(i11)
	vel(i11,2)=soln(i22)
	pres(i11)=soln(i33)

5165	continue



c     Assiging the nodal values to 4-node element mesh nodal points
c     ===============================================================================



	do 7017 iel = 1,nel
		
		iii1 = iabs(node(iel,1))
		iii2 = iabs(node(iel,3))
		iii3 = iabs(node(iel,5))
		iii4 = iabs(node(iel,7))

		jjj1 = iabs(rode(iel,1))
		jjj2 = iabs(rode(iel,2))
		jjj3 = iabs(rode(iel,3))
		jjj4 = iabs(rode(iel,4))


		rel(jjj1,1) = vel(iii1,1)
		rel(jjj1,2) = vel(iii1,2) 
		qres(jjj1)  = pres(iii1) 

		


		bbb1 = jjj1 + qnp
		ccc1 = jjj1 + 2*qnp

		roln(jjj1) = rel(jjj1,1) 
		roln(bbb1) = rel(jjj1,2) 
		roln(ccc1) = qres(jjj1)  
		
						


		rel(jjj2,1) = vel(iii2,1) 
		rel(jjj2,2) = vel(iii2,2) 
		qres(jjj2)  = pres(iii2) 


		bbb2 = jjj2 + qnp
		ccc2 = jjj2 + 2*qnp

		roln(jjj2) = rel(jjj2,1) 
		roln(bbb2) = rel(jjj2,2) 
		roln(ccc2) = qres(jjj2) 



		rel(jjj3,1) = vel(iii3,1) 
		rel(jjj3,2) = vel(iii3,2) 
		qres(jjj3)  = pres(iii3)  


		bbb3 = jjj3 + qnp
		ccc3 = jjj3+ 2*qnp

		roln(jjj3) = rel(jjj3,1)  
		roln(bbb3) = rel(jjj3,2) 
		roln(ccc3) = qres(jjj3) 


		rel(jjj4,1) = vel(iii4,1) 
		rel(jjj4,2) = vel(iii4,2) 
		qres(jjj4)  = pres(iii4) 

		bbb4 = jjj4 + qnp
		ccc4 = jjj4+ 2*qnp

		roln(jjj4) = rel(jjj4,1) 
		roln(bbb4) = rel(jjj4,2) 
		roln(ccc4) = qres(jjj4) 



7017	end do


c	=========================================================================================================================

	



c     ================
c     Print the output
c     ================

      if(iter.eq.1) goto 87777
      iiter=(iter/ntep)*ntep
	if(iiter.eq.iter) goto 87777
	                  goto 87778
					  
87777	call output
     1     (nnp,qnp,soln, roln,maxdf, maxnp, icord)

c	do 7029 inp = 1,qnp
c				  jnp = inp + qnp
c				  mnp = jnp + qnp
c	write(101,7000)inp,roln(inp),roln(jnp),roln(mnp)

c7029	continue

87778 continue
	

c     ================================================================
c                           Convergence check
c     ================================================================
 
      call contol
	1      (soln  ,  iter   ,  ntov   ,  nnp   ,  maxnp ,
     2       maxdf,   errov  ,  vet    )					


5150  continue


c	=================================================================

	call tecplot
	
	1(nnp, nel, qnp, qel, maxel, maxst, maxnp, ndim,
	2 vel, pres, rel, qres, cord, node, gord, rode) 






      close(      51)
      close(unit= 60)
      close(unit= 11)
      close(unit= 14)
      close(unit= 15)
      close(      20)
	close(unit= 71)
	close(unit= 81)
	close(unit= 101)
	close(unit= 301)
	close(unit= 401)

c
c    ===================================================================
c                         Read statements
c    ===================================================================
c
c
2000  format(a     )
2010  format(80a   )
2020  format(3i5   )
2030  format(4i5   )
2040  format(2i5   )
2060  format(2f10.5)
2070  format(2d10.5)
2859	format(2i5)
c
c
c    ===================================================================
c                         Write statements
c    ===================================================================

4010  format(' ',5(/),' ',20x,60('*'),/' ',20x,'*',58x,'*',/
     1' ',20x,'*','  A  two  dimensional finite element model of a  ',
     29x,'*',/' ',20x,'*','  non-newtonian isothermal flow using ',
     320x,'*',/' ',20x,'*','  the UVP method.  ',39x,'*',/' ',20x,'*',
     558x,'*',/' ',20x,60('*')///,' ',20x,80('-'),/' ',20x,80a,/' ',
     620x,80('-'),///)
c
4020  format(' ',20x,3('['),' element description data',10('.'),/
     125x,'no.of nodes per element                  =',i10,/
     225x,'no.of integration points                 =',i10,/
     3//)
c
4030  format('    *** coordinate system is cartesian (planar) ***')
4040  format('*** coordinate system is cylindrical(axisymmetric) ***')
4050  format(' ')
4060  format(' ',10('['),'input data unacceptable',10(']')///)
c
4070  format(' ',20x,3('['),' mesh description data ',10('.'),/
     125x,'no.of nodal points                       =',i10,/
     225x,'no.of elements                           =',i10,/
     325x,'no.of nodal constraints on boundary      =',i10,/
     425x,'no.of different materials                =',i10,//)
c
4080  format(' ',20x,3('['),' uniform body force vector ',10('.'),/
     125x,'grav1                                    =',f15.4,/
     225x,'grav2                                    =',f15.4,//)
4090  format(///'     time step    ',i5,//)

7000	format(i5,f10.6,f10.6,e20.6)
7011	format(5i8)


c    ===================================================================
c                         print statements
c    ===================================================================

1101	format(' ',/)
1111	format (/'  CFDYNAMICS simulates fluid dynamical behaviour in the 
     1  ',/' coupled free/porous flow regimes')
1121	format(' ', 2(/))

1131  format("  *************************************************
     1*****************")


c     ===========
      END PROGRAM 
c     ===========
c
c     ====================================
c     SUBROUTINES USED IN THE CALCULATIONS   
c     ====================================  
c
c
c     ==================================================================
      subroutine bacsub
     1    (ntotl,ifix ,vfix ,rhs  ,soln ,mfrnt ,rwork,iwork,idv2)
     
c
c
      implicit double precision(a-h,o-z)
      dimension ifix (ntotl),vfix (ntotl),rhs (ntotl),soln (ntotl)
      dimension rwork(mfrnt),iwork(mfrnt)
c
c
c 
                         do 6010 i=1,ntotl
                         soln(i) =0.0
                         if(ifix(i).ne.0) soln(i)=vfix(i)
 6010                    continue
                         
	                   
 
      do 6020 kp=1,ntotl
c
      backspace idv2
      read(idv2)     i,ifrnt,jfrnt,(iwork(k),rwork(k),k=1,ifrnt)
      backspace idv2
c
      if(ifix(i).ne.0)  go to 6020
c
      ww           = 0.0
      rwork(jfrnt) = 0.0
c
                         do 6030 k=1,ifrnt
                         j=iabs(iwork(k))
                         ww  =ww - rwork(k)*soln(j)
6030                     continue
      soln (i)=rhs(i)+ww
6020  continue

      return
      end
c
c
c    ==================================================================
      subroutine clean
     1    (ncn  ,   nel   ,   ndf  ,   node , rode,  r1 , maxel, maxst,
     2     maxdf,   bc    ,   ncod ,   icho                            ) 
c
      implicit double precision(a-h,o-z)
c
c     arguments
c     =========
c     all arguments are defined elsewhere.
c
      dimension r1   (maxdf) ,   node(maxel,maxst)
      dimension bc   (maxdf) ,   ncod(maxdf      )
	integer rode (maxel,maxst)

	ntrix=27
c
c     function
c     ========
c     cleans the used arrays and makes them ready for  solution
c
           do  6010 i   = 1,maxdf
                  r1(i) = 0.0
                  bc(i) = 0.0
                ncod(i) = 0
6010                   continue
                ntrix   = ndf*ncn
           do  6020 iel = 1,nel
           do  6020 inp = 1,ntrix
           node(iel,inp) = iabs(node(iel,inp))
c		 rode(iel,inp) = iabs(rode(iel,inp))
6020                   continue
           if(icho.ne.1)then
           do 6030 iel = 1,nel
           write(11,3010)iel,(node(iel,j),j=1,ncn)
3010       format(10i5)  
6030       continue
           do 6040 jel = 1,nel
           write(11,3020)jel,(rode(jel,j),j=1,ncn)
3020       format(5i5)  
6040       continue
		
           endif
c
      return
      end
c
c    ===================================================================
      subroutine contol
	1      (soln  ,  iter   ,  ntov   ,  nnp   ,  maxnp ,
     2       maxdf,   errov  ,  vet    )					
     
c
      implicit double precision(a-h,o-z)
c
      dimension soln (maxdf), vet (maxdf) 
      
	errv  = 0.0
      torv  = 0.0
	nttt  = nnp*2
		
c    calculate difference between velocities in consecutive iterations
c    =================================================================
c
                     do 6010 icheck = 1,nttt
      if(iter.eq.1)  vet(icheck) = 0.0
                     errv = errv + (soln(icheck)-vet(icheck)) ** 2
                     torv = torv +  soln(icheck)**2
c
      vet(icheck) = soln(icheck)
c
6010                 continue
                     errov= errv/torv
      return
      end
c
c     =================================================================
      subroutine deriv
     1    (iel  ,   ig   ,   jg   ,   p    ,   del  ,   b    ,   ncn  ,
     2     da   ,   cg   ,   node ,   cord ,   maxel,   maxnp,   maxst)
c
      implicit double precision(a-h,o-z)
c
      dimension p   (9        ), b   (2,    9)        
	dimension del (2,      9), cg  (3      )         
	dimension cj  (2,      2), cji (2,    2)
      dimension node(maxel, 27), cord(maxnp,2)
c
      do 6010 j=1,2
      do 6010 l=1,2
      
	gash=0.0
      
	do 6020 k=1,ncn
      
	a=iabs(node(iel,k))
6020  gash=gash + del(j,k)*cord(a,l)
6010  cj(j,l)=gash 
      detj=cj(1,1)*cj(2,2)- cj(1,2)*cj(2,1)
      
	if(detj.le.0.0) then
      
	write(60,3010) iel,detj
3010  format(1x ,' Error: Zero or Negative Jacobian. ', i6,g20.5)
      stop 
      
	endif
c
      cji(1,1) = cj(2,2) / detj
      cji(1,2) =-cj(1,2) / detj
      cji(2,1) =-cj(2,1) / detj
      cji(2,2) = cj(1,1) / detj
c
      do 6030 j=1,2
      do 6030 l=1,ncn
      
	b(j,l)=0.0
      
	do 6030 k=1,2
6030  b(j,l) = b(j,l) + cji(j,k) * del(k,l)
      
	da= detj*cg(ig)*cg(jg)
c
      return
      end
c
c    ==================================================================
      subroutine flow  
     1		  (node  ,   cord ,   pmat ,   nopp ,   mdf  ,   ndn     ,   
	1           ncod  ,   bc   ,   vel  ,   pres ,   r1   ,   ldest   ,
     2           kdest ,   nk   ,   eq   ,   lhed ,   khed ,
     3           kpiv  ,   lpiv ,   jmod ,   qq   ,   pvkol,   iter    ,
     4           nel   ,   ncn  ,   ngaus,   grav1,   permx,   permy   ,
     5           grav2 ,   p    ,   del  ,   b    ,   ntrix,   maxel   ,
     6           maxnp ,   maxst,   maxfr,   maxdf,   ndim ,   aa      ,
     7           xg    ,   da   ,   ntov ,   icord,   idv4 ,   sinv    ,
     8           nnp   ,   pp   ,   delp ,   soln ,   num              )    	                       
	
c
      implicit double precision(a-h,o-z)
c
      dimension node (maxel,maxst),pmat (maxel,   9),cord (maxnp, ndim)
      dimension ncod (maxdf      ),bc   (maxdf     ),sinv (maxel,  ncn)
      dimension vel  (nnp,   ndim),r1   (maxdf     )
      dimension aa   (maxst,maxst),rr   (maxst     ),ldest(maxst      )
      dimension xg   (          3),cg   (         3),kdest(maxst      )
      dimension x    (          2),v    (         2),nk   (maxst      )
      dimension bicn (          2),hh   (         2),b    (    2,    9)
      dimension p    (          9),del  (    2,   9)
      dimension eq   (maxfr,maxfr),nopp (maxdf     )     
      dimension lhed (maxfr      ),khed (maxfr     ),jmod (maxfr      )
      dimension lpiv (maxfr      ),kpiv (maxfr     ),qq   (maxfr      )
      dimension pvkol(maxfr      ),mdf  (maxdf     ),ndn  (maxdf      )
	dimension ak   (maxst,maxst),akf  (maxst     )   
      dimension pres (maxnp      )
	dimension pp   (          4)
	dimension delp (    2,    4)
	dimension ppp  (          9)
	dimension soln (      maxdf)
	dimension aa01 (maxst,maxst)
	dimension aa02 (maxst,maxst)
c

      do 5001 iel=1,nel
	 
                                                 rvisc = pmat(iel,1)
                                                 rpef  = pmat(iel,2)
                                                 power = pmat(iel,3)
                                                 rtem  = pmat(iel,4)
                                                 tbco  = pmat(iel,5)
                                                 taco  = pmat(iel,6)
                                                 roden = pmat(iel,8)
											   gamad = pmat(iel,9) 
                                          	    

                 do 6010 idf= 1,ntrix
                 rr (idf)      = 0.0
                 
			   do 6010 jdf= 1,ntrix
                 aa   (idf,jdf)= 0.0
	           aa01 (idf,jdf)= 0.0
	           aa02 (idf,jdf)= 0.0
 6010            continue
c
c     Numerical Integration loop
c     ==========================
                               
      call gaussp(ngaus,xg,cg)
c
	       lg=0

      do 6020 ig=1,ngaus
                                    g = xg(ig)
      do 6020 jg=1,ngaus
                                    h = xg(jg)
                            
						         lg = lg + 1
   
      if(iter.eq.1) then

      call shape   (g     ,h      ,p      ,del   ,   9)
	call shape   (g     ,h      ,pp     ,delp  ,   4)
      ncn=9



      call deriv (iel   ,ig     ,jg     ,p     ,del    ,b       ,ncn 
	1           ,da    ,cg     ,node   ,cord  ,maxel  ,maxnp   ,maxst)

	              iig=ig
	              jjg=jg
        
                    write(15)   iel ,ig ,jg ,p ,del ,b ,da, pp
      else

      if(.not. eof(15))read(15) iiel,iig,jjg,p ,del ,b , da, pp
      endif


	ppp(1)=pp(1)
	ppp(2)=0.0
	ppp(3)=pp(2)
	ppp(4)=0.0
	ppp(5)=pp(3)
	ppp(6)=0.0
	ppp(7)=pp(4)
	ppp(8)=0.0
	ppp(9)=0.0
c
c     calculation of viscosity based on the constitutive equation
c     ===========================================================

      spress = 0.0
      stemp  = 0.0
c
      
	          do 5333 ip = 1,ncn
                jp = iabs(node(iel,ip))
 5333           continue
                epsii = 1.d-10
                gamad = sinv(iel,lg)
                if(gamad.lt.epsii) gamad = epsii
c      
      call visca
     1(rvisc,power,visc,stemp,rtem,tbco,spress,rpef,taco,gamad)
                
c     prepration of the convective acceleration terms/balancing dissipation is used
c     -----------------------------------------------------------------------------
c
 
                       do 6050 idff= 1,2
                       x(idff)     = 0.0
                       v(idff)     = 0.0
6050                   continue
 

                       do 6060 icn = 1 ,ncn
                       jcn = iabs(node(iel,icn))
                       do 6060 idff= 1 ,  2
                       x(idff) = x(idff) + p(icn)*cord(jcn,idff)
                       v(idff) = v(idff) + p(icn)*vel (jcn,idff)
6060	 	             continue


      if(icord.eq.1) then
c
c     modify da for axisymmetric computations.
c
      da = da * x(1)
 
      endif

c     =====================================================================
c     W   O   R   K   I   N   G          E   Q   U   A   T   I   O   N   S  
c     =====================================================================
c     
c     Stokes Equations 
c     ================

    
      if(iel.ge.1.and.iel.le.1560 .or. iel.ge.1681.and.iel.le.5600) then

     
      do 6070 i=1,ncn

       			   j11= i
                     j12= i + ncn
	               j13= i + 2*ncn
      
      do 6070 j=1,ncn
                     j21= j
                     j22= j + ncn
	               j23= j + 2*ncn



      aa(j11,j21)=aa(j11,j21) +  2.0*visc*b(1,i)*b(1,j)*da
	1                        +  visc*b(2,j)*b(2,i)*da
                     
      aa(j11,j22)=aa(j11,j22) +  visc*b(2,i)*b(1,j)*da
 
      aa(j11,j23)=aa(j11,j23) -  ppp(j)*b(1,i)*da
     
                       
      aa(j12,j21)=aa(j12,j21) +  visc*b(1,i)*b(2,j)*da
         
      aa(j12,j22)=aa(j12,j22) +  visc*b(1,i)*b(1,j)*da
	1                        +  2*visc*b(2,j)*b(2,i)*da
     
   
	aa(j12,j23)=aa(j12,j23) -  ppp(j)*b(2,i)*da


      aa(j13,j21)=aa(j13,j21) -  ppp(i)*b(1,j)*da

	aa(j13,j22)=aa(j13,j22) -  ppp(i)*b(2,j)*da
      
	aa(j13,j23)=aa(j13,j23) -  0.0
    
	
                                              
6070  continue
      end if

c     Darcy Equations 
c     ===============


      if(iel.ge.1601  .and. iel.le.1640) then
     
         
      do 6080 i=1,ncn

       			   j11= i
                     j12= i + ncn
	               j13= i + 2*ncn
      
      do 6080 j=1,ncn
                     j21= j
                     j22= j + ncn
	               j23= j + 2*ncn

     
      aa(j11,j21) = aa(j11,j21) + (visc*p(i)*p(j)*da)/permx 
	
      aa(j11,j22) = aa(j11,j22) +  0.0
 
      aa(j11,j23) = aa(j11,j23) -  ppp(j)*b(1,i)*da
     
      aa(j12,j21) = aa(j12,j21) +  0.0
         
      aa(j12,j22) = aa(j12,j22) +  (visc*p(i)*p(j)*da)/permy
	  
   	aa(j12,j23) = aa(j12,j23) -  ppp(j)*b(2,i)*da

      aa(j13,j21) = aa(j13,j21) -  ppp(i)*b(1,j)*da

	aa(j13,j22) = aa(j13,j22) -  ppp(i)*b(2,j)*da
      
	aa(j13,j23) = aa(j13,j23) -  0 .0 
	
                                              
6080  continue
      end if

c     ======================================
c     Elements on the Free/Porous Interface
c     ======================================

      if(iel.ge.1561  .and. iel.le.1600) then	
     
      
c     Replacement of the nodes on the interface by the Darcy working equations

      do 6090 i=1,ncn

       			   j11= i
                     j12= i + ncn
	               j13= i + 2*ncn
      
 
      do 6090 j=1,ncn
                     j21= j
                     j22= j + ncn
	               j23= j + 2*ncn
     

c     Porous flow calculation 
c     =======================
c
c     --- Stiffness Matrix of Left Hand Side -------------------------------------------------
c
c     For Transient state (Cartesian co-ordinate system) 

	
      aa02(j11,j21)=aa02(j11,j21) +  (visc*p(i)*p(j)*da)/permx
	
      aa02(j11,j22)=aa02(j11,j22) +  0.0
 
      aa02(j11,j23)=aa02(j11,j23) -  ppp(j)*b(1,i)*da
     
      aa02(j12,j21)=aa02(j12,j21) +  0.0
         
      aa02(j12,j22)=aa02(j12,j22) +  (visc*p(i)*p(j)*da)/permy
	  
   	aa02(j12,j23)=aa02(j12,j23) -  ppp(j)*b(2,i)*da

      aa02(j13,j21)=aa02(j13,j21) -  ppp(i)*b(1,j)*da

	aa02(j13,j22)=aa02(j13,j22) -  ppp(i)*b(2,j)*da
      
	aa02(j13,j23)=aa02(j13,j23) -  0.0 	
     

c     Free flow calculation
c     =====================


      aa01(j11,j21)=aa01(j11,j21) +  2.0*visc*b(1,i)*b(1,j)*da
	1                        +  visc*b(2,j)*b(2,i)*da
                     
      aa01(j11,j22)=aa01(j11,j22) +  visc*b(2,i)*b(1,j)*da
 
      aa01(j11,j23)=aa01(j11,j23) -  ppp(j)*b(1,i)*da
     
                       
      aa01(j12,j21)=aa01(j12,j21) +  visc*b(1,i)*b(2,j)*da
         
      aa01(j12,j22)=aa01(j12,j22) +  visc*b(1,i)*b(1,j)*da
	1                        +  2*visc*b(2,j)*b(2,i)*da
     
   
	aa01(j12,j23)=aa01(j12,j23) -  ppp(j)*b(2,i)*da


      aa01(j13,j21)=aa01(j13,j21) -  ppp(i)*b(1,j)*da

	aa01(j13,j22)=aa01(j13,j22) -  ppp(i)*b(2,j)*da
      
	aa01(j13,j23)=aa01(j13,j23) -  0.0
  

c     ====================================================
c     Replacement of Stokes with Darcy on iterfacial nodes
c     ====================================================
    

      
	if (j11.eq.3  .or. j11.eq.4  .or. j11.eq.5 .or. 
	1    j12.eq.12 .or. j12.eq.13 .or. j12.eq.14.or. 
     2    j13.eq.21 .or. j13.eq.22 .or. j13.eq.23    ) then

	aa(j11,j21) =  aa02(j11,j21)  
	aa(j11,j22) =  aa02(j11,j22)  
	aa(j11,j23) =  aa02(j11,j23)  

	aa(j12,j21) =  aa02(j12,j21)  
	aa(j12,j22) =  aa02(j12,j22)  
	aa(j12,j23) =  aa02(j12,j23)  

	aa(j13,j21) =  aa02(j13,j21)  
	aa(j13,j22) =  aa02(j13,j22)  
	aa(j13,j23) =  aa02(j13,j23)  


	else

	aa(j11,j21) =  aa01(j11,j21)  
	aa(j11,j22) =  aa01(j11,j22)  
	aa(j11,j23) =  aa01(j11,j23)  

	aa(j12,j21) =  aa01(j12,j21)  
	aa(j12,j22) =  aa01(j12,j22)  
	aa(j12,j23) =  aa01(j12,j23)  

	aa(j13,j21) =  aa01(j13,j21)  
	aa(j13,j22) =  aa01(j13,j22)  
	aa(j13,j23) =  aa01(j13,j23)  

	endif
 
6090  continue
      end if



c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



      if (iel.ge.1641 .and. iel.le.1680) then
     
      
c     Replacement of the nodes on the interface by the Darcy working equations

      do 7383 i=1,ncn

       			   j11= i
                     j12= i + ncn
	               j13= i + 2*ncn
      
 
      do 7383 j=1,ncn
                     j21= j
                     j22= j + ncn
	               j23= j + 2*ncn
     

c     Porous flow calculation 
c     =======================
c
c     --- Stiffness Matrix of Left Hand Side -------------------------------------------------
c
c     For Transient state (Cartesian co-ordinate system) 

	
      aa02(j11,j21)=aa02(j11,j21) +  (visc*p(i)*p(j)*da)/permx
	
      aa02(j11,j22)=aa02(j11,j22) +  0.0
 
      aa02(j11,j23)=aa02(j11,j23) -  ppp(j)*b(1,i)*da
     
      aa02(j12,j21)=aa02(j12,j21) +  0.0
         
      aa02(j12,j22)=aa02(j12,j22) +  (visc*p(i)*p(j)*da)/permy
	  
   	aa02(j12,j23)=aa02(j12,j23) -  ppp(j)*b(2,i)*da

      aa02(j13,j21)=aa02(j13,j21) -  ppp(i)*b(1,j)*da

	aa02(j13,j22)=aa02(j13,j22) -  ppp(i)*b(2,j)*da
      
	aa02(j13,j23)=aa02(j13,j23) -  0.0 	
     

c     Free flow calculation
c     =====================


      aa01(j11,j21)=aa01(j11,j21) +  2.0*visc*b(1,i)*b(1,j)*da
	1                        +  visc*b(2,j)*b(2,i)*da
                     
      aa01(j11,j22)=aa01(j11,j22) +  visc*b(2,i)*b(1,j)*da
 
      aa01(j11,j23)=aa01(j11,j23) -  ppp(j)*b(1,i)*da
     
                       
      aa01(j12,j21)=aa01(j12,j21) +  visc*b(1,i)*b(2,j)*da
         
      aa01(j12,j22)=aa01(j12,j22) +  visc*b(1,i)*b(1,j)*da
	1                        +  2*visc*b(2,j)*b(2,i)*da
     
   
	aa01(j12,j23)=aa01(j12,j23) -  ppp(j)*b(2,i)*da


      aa01(j13,j21)=aa01(j13,j21) -  ppp(i)*b(1,j)*da

	aa01(j13,j22)=aa01(j13,j22) -  ppp(i)*b(2,j)*da
      
	aa01(j13,j23)=aa01(j13,j23) -  0.0
  

c     ====================================================
c     Replacement of Stokes with Darcy on iterfacial nodes
c     ====================================================
    

      
	if (j11.eq.1  .or. j11.eq.7  .or. j11.eq.8  .or.
	1    j12.eq.10 .or. j12.eq.16 .or. j12.eq.17 .or.
	2	j13.eq.19 .or. j13.eq.25 .or. j13.eq.26 ) then

	aa(j11,j21) =  aa02(j11,j21)  
	aa(j11,j22) =  aa02(j11,j22)  
	aa(j11,j23) =  aa02(j11,j23)  

	aa(j12,j21) =  aa02(j12,j21)  
	aa(j12,j22) =  aa02(j12,j22)  
	aa(j12,j23) =  aa02(j12,j23)  

	aa(j13,j21) =  aa02(j13,j21)  
	aa(j13,j22) =  aa02(j13,j22)  
	aa(j13,j23) =  aa02(j13,j23)  


	else

	aa(j11,j21) =  aa01(j11,j21)  
	aa(j11,j22) =  aa01(j11,j22)  
	aa(j11,j23) =  aa01(j11,j23)  

	aa(j12,j21) =  aa01(j12,j21)  
	aa(j12,j22) =  aa01(j12,j22)  
	aa(j12,j23) =  aa01(j12,j23)  

	aa(j13,j21) =  aa01(j13,j21)  
	aa(j13,j22) =  aa01(j13,j22)  
	aa(j13,j23) =  aa01(j13,j23)  

	endif
 
7383  continue
      end if


6020  continue


c     ==================================================
c     Writing the Stiffness Matrix for Element Number 20
c     ==================================================  

         if (iel==20)then
	
	write (idv4,3045) 
	write (idv4,3050) iter
      write (idv4,3055) ((aa(i,j),j=1,ntrix),i=1,ntrix)
         
3045  format(///,' ',10('*'),' element stiffness matrix for element 
     120',10('*'),///)
3050  format(///,' ','  time step      =',i5,///) 
3055  format(27(E15.8,3x))
      
         end if 



c     ===========================================================
c     Checking the element for which the stiffness matrix is zero
c     ===========================================================  
      
      icount =0.0
	 
	do 1111  i=1,ntrix
	do 1111  j=1,ntrix
	
	if(aa(i,j) .eq. 0) then
	icount=icount+1
	end if

1111	continue

      if (icount.eq.729) print*,iel


c     Evaluation of Elemental Load Vector
c     ===================================
   
      do 6085 i=1,ncn

       			   j11= i
                     j12= i + ncn
	               j13= i + 2*ncn
      
     
	rr(j11)=0.0
      rr(j12)=0.0
	rr(j13)=0.0

 6085 continue 

      maxte=maxdf

      call front
     1          (aa   ,   rr   ,   iel   ,   node ,   maxel,   maxst,  
     2           ldest,   kdest,   nk    ,   maxfr,   eq   ,   lhed ,
     3           khed ,   kpiv ,   lpiv  ,   jmod ,   qq   ,   pvkol,
     4           r1   ,   ncod ,   bc    ,   nopp ,   mdf  ,
     5           ndn  ,   maxdf,   nel   ,   maxte,   ntov ,   soln )
 
 5001 continue

      return
      end
c
c    =================================================================
      subroutine front
     1    (aa   ,   rr   ,   iel  ,   nop  ,   maxel,   maxst,   ldest,   
     2     kdest,   nk   ,   maxfr,   eq   ,   lhed ,   khed ,   kpiv ,
     3     lpiv ,   jmod ,   qq   ,   pvkol,   r1   ,   ncod ,
     4     bc   ,   nopp ,   mdf  ,   ndn  ,   maxdf,   nel  ,   maxte,
     5     ntov ,   soln                                              ) 
 
c     Frontal elimination routine using diagonal pivoting
c
      implicit double precision(a-h,o-z)
c

      dimension aa   (maxst,maxst) ,rr   (maxst)
      dimension nop  (maxel,maxst)
      dimension ldest(maxst)       ,kdest(maxst)       ,nk   (maxst)
      dimension eq   (maxfr,maxfr) ,lhed (maxfr)       ,khed (maxfr)
      dimension kpiv (maxfr)       ,lpiv (maxfr)
      dimension jmod (maxfr)       ,qq   (maxfr)       ,pvkol(maxfr)
      dimension r1   (maxdf)       ,ncod (maxdf)
      dimension bc   (maxdf)       ,nopp (maxdf)       ,mdf  (maxdf)
      dimension ndn  (maxdf)  
	dimension soln (maxte)     
	
c
c
c     nlp and ndl are the file specifiers for units 60 and 14 respectively
c     ====================================================================

      nlp=60
      nd1=14

c
c     Prefront
c     ========

      if(iel.eq.1) nell = 0
      if(iel.eq.1) ntra = 1
      if(ntra.eq.0) goto 6040
      nmax = maxfr     
      ntra = 0
      ncrit = 20       
      lfron = 0
      nlarg = nmax-10  
c
c     Find last appeareance of each node
c     ==================================
      nlast = 0
      do 6010 i = 1,ntov
      do 6020 n = 1,nel
      jdn = ndn(n)
      do 6030 l = 1,jdn
      if(nop(n,l).ne.i)go to 6030
      nlast1 = n
      nlast = n
      l1 = l
6030  continue
6020  continue
      if(nlast.eq.0) go to 6010
      nop(nlast,l1) = -nop(nlast,l1)
      nlast = 0
6010  continue
      ntrix = jdn
c
c     Assembly
c     ========   
6040  continue
      if(iel.gt.1) go to 6060
      lcol = 0
      do 6050 i = 1,nmax
      do 6050 j = 1,nmax
      eq(j,i) = 0
6050  continue
6060  nell = nell+1
      n = nell
      jdn = ndn(nell)
      kc = 0
      do 6070 j = 1,jdn 
      nn = nop(n,j)
      m = iabs(nn)
      k = nopp(m)
      idf = mdf(m)
      r1(m) = rr(j)+r1(m)
      do 6070 l = 1,idf
      kc = kc+1
      ii = k+l-1
      if(nn.lt.0)ii = -ii
      nk(kc) = ii
6070  continue
c
c     Set up heading vectors
c     ======================
c
      do 6080 lk = 1,kc
      node = nk(lk)
      if(lcol.eq.0)goto 6100
      do 6090 l = 1,lcol
      ll = l
      if(iabs(node).eq.iabs(lhed(l)))go to 6110
6090  continue
6100  lcol = lcol+1
      ldest(lk) = lcol
      lhed(lcol) = node
      go to 6080
6110  ldest(lk) = ll
      lhed(ll) = node
6080  continue
      if(lcol.le.nmax)go to 6130
      nerror = 2
      write(nlp,3010)nerror
      stop
6130  continue
      do 6140 l = 1,kc
      ll = ldest(l)
      do 6140 k = 1,kc
      kk = ldest(k)
      eq(kk,ll) = eq(kk,ll)+aa(k,l)
6140  continue
      if(lcol.lt.ncrit.and.nell.lt.nel) return
c
c     Find out which matrix elements are fully assembeled
c     ===================================================
6150  lc = 0
      ir = 0
      do 6160 l = 1,lcol
      kt = lhed(l)
      if(kt.ge.0)go to 6160
      lc = lc+1
      lpiv(lc) = l
      kro = iabs(kt)
      if(ncod(kro).ne.1)go to 6160
      ir = ir+1
      jmod(ir) = l
      ncod(kro) = 2
      r1(kro) = bc(kro)
6160  continue
c
c     Modify equations with applied boundary conditions
c     =================================================
      if(ir.eq.0)go to 6190
      do 6170 irr = 1,ir
      k = jmod(irr)
      kh = iabs(lhed(k))
      do 6180 l = 1,lcol
      eq(k,l) = 0.
      lh = iabs(lhed(l))
      if(lh.eq.kh)eq(k,l) = 1.
6180  continue
6170  continue
6190  continue
      if(lc.gt.0)go to 6200
      ncrit = ncrit+10
      write(nlp,3020)ncrit
      if(ncrit.le.nlarg) return
      nerror = 3
      write(nlp,3030)nerror
      stop
6200  continue
c
c     Search for absolute pivot
c     =========================
      pivot = 0.
      do 6210 l = 1,lc
      lpivc = lpiv(l)
      kpivr = lpivc
      piva = eq(kpivr,lpivc)
      if(abs(piva).lt.abs(pivot))go to 6220
      pivot = piva
      lpivco = lpivc
      kpivro = kpivr
6220  continue
6210  continue


      if(pivot.eq.0.0) return
      
	
c
c     Normalise pivotal row
c     =====================
      lco = iabs(lhed(lpivco))
      kro = lco

c     if(nit.eq.0.or.npra.eq.0)go to 6230
c6230 continue

      if(abs(pivot).lt.0.1d-28) write(nlp,3050)
      do 6240 l = 1,lcol
      qq(l) = eq(kpivro,l)/pivot
6240  continue
      rhs = r1(kro)/pivot
      r1(kro) = rhs
      pvkol(kpivro) = pivot
c
c     Eliminate then delete pivotal row and column
c     ============================================
      if(kpivro.eq.1)go to 6300
      kpivr = kpivro-1
      do 6250 k = 1,kpivr
      krw = iabs(lhed(k))
      fac = eq(k,lpivco)
      pvkol(k) = fac
      if(lpivco.eq.1.or.fac.eq.0.)go to 6270
      lpivc = lpivco-1
      do 6260 l = 1,lpivc
      eq(k,l) = eq(k,l)-fac*qq(l)
6260  continue
6270  if(lpivco.eq.lcol)go to 6290
      lpivc = lpivco+1
      do 6280 l = lpivc,lcol
      eq(k,l-1) = eq(k,l)-fac*qq(l)
6280  continue
6290  r1(krw) = r1(krw)-fac*rhs
6250  continue
6300  if(kpivro.eq.lcol)go to 6360
      kpivr = kpivro+1
      do 6310 k = kpivr,lcol
      krw = iabs(lhed(k))
      fac = eq(k,lpivco)
      pvkol(k) = fac
      if(lpivco.eq.1)go to 6330
      lpivc = lpivco-1
      do 6320 l = 1,lpivc
      eq(k-1,l) = eq(k,l)-fac*qq(l)
6320  continue
6330  if(lpivco.eq.lcol)go to 6350
      lpivc = lpivco+1
      do 6340 l = lpivc,lcol
      eq(k-1,l-1) = eq(k,l)-fac*qq(l)
6340  continue
6350  r1(krw) = r1(krw)-fac*rhs
6310  continue
6360  continue
c
c     Write pivotal equation on disc
c     ==============================   
      write(nd1) kro,lcol,lpivco,(lhed(l),qq(l),l = 1,lcol)
      do 6370 l = 1,lcol
      eq(l,lcol) = 0.
      eq(lcol,l) = 0.
6370  continue
c
c     Rearrange heading vectors
c     =========================
      lcol = lcol-1
      if(lpivco.eq.lcol+1)go to 6390
      do 6380 l = lpivco,lcol
      lhed(l) = lhed(l+1)
6380  continue
6390  continue
c
c     Determine whether to assemble,eliminate,or backsubstitute
c     =========================================================
      if(lcol.gt.ncrit)go to 6150
      if(nell.lt.nel) return
      if(lcol.gt.1)go to 6150
      lco = iabs(lhed(1))
      kpivro = 1
      pivot = eq(1,1)
      kro = lco
      lpivco = 1
      qq(1) = 1.

c     if(nit.eq.0.or.npra.eq.0)go to 6400
c     write(nlp,3040)lco,kro,pivot

      if(abs(pivot).lt.1d-28)go to 6410

c6400 continue

      r1(kro) = r1(kro)/pivot
      write(nd1) kro,lcol,lpivco,lhed(1),qq(1)
c
c     start back-substitution
c     =======================
      call bacsub
     1     (ntov ,    ncod ,    bc   ,   r1 ,   soln,    
     2      maxfr,    qq   ,    lhed ,   nd1      )
c
c     main exit with solution
c     =======================
c
6410  continue
c
c
3010  format(/'  nerror=',i5//
     1 '  the difference nmax-ncrit is not sufficiently large'
     1/'  to permit the assembly of the next element---'
     1/'  either increase nmax or lower ncrit'
     1/)
3020  format('  frontwidth value=',i4)
3030  format(/'  nerror=',i5//
     1 '  there are no more rows fully summed,this may be due to---'
     1/'  (1)incorrect coding of nop or nk arrays'
     1/'  (2)incorrect value of ncrit. increase ncrit to permit'
     1/'     whole front to be assembled'
     1/)

c3040  format(13h pivotal row=,i4,16h pivotal column=,i4,7h pivot=,e20.10
c    1)

3050  format('  warning-matrix singular or ill conditioned')

      return
      end
c
c
c    ================================================================   
      subroutine gaussp (ngaus ,  xg ,  cg)
c
      implicit double precision (a-h,o-z)
c
c     x(g)   specifies the coordinates of the Gauss points
c     c(g)   specifies the Gauss weights
c
      dimension xg(3),cg(3)

      if(ngaus.eq.1) then
      xg(1)=0.0
      cg(1)=2.0

      elseif(ngaus.eq.2)  then
      xg(1) = 0.57735026919d00
      xg(2) = -xg(1)
      cg(1) = 1.00
      cg(2) = 1.00
      
	else
      xg(1) = 0.77459666924d00
      xg(2) = 0.0
      xg(3) = -xg(1)
      cg(1) = 0.55555555556d00
      cg(2) = 0.88888888889d00
      cg(3) = cg(1)

      endif
      return
      end
c 
c   
c     ==================================================================
      subroutine getbcd (nbc ,  ibc ,  jbc , vbc,  idv1 , idv2  , maxbc)
c
      implicit double precision(a-h,o-z)
c
      dimension ibc  (maxbc) ,jbc  (maxbc),  vbc  (maxbc)

c
c     arguments
c     =========
c     nbc    number of nodal constraint data
c     ibc    array for constrained nodal points
c     jbc    array for constrained degree of freedom
c     vbc    array for boundary values
c     udv1   input device id.
c     idv2   output device id. 
c     maxbc  see below
c
      if (.not. eof(51))read (idv1,1010) (ibc(ind) ,jbc(ind) ,vbc(ind)
	1                                   ,ind=1,nbc)
	print*, "boundary conditions array read" 
      write(idv2,3010)
      write(idv2,3020)                   (ibc(ind) ,jbc(ind) ,vbc(ind)
	1                                   ,ind=1,nbc)
c
      return
c
1010  format(2i5,f10.4)
3010  format(' ',//  /,' ',20('*'),' nodal constraint ',20('*'),//
     1' ',(8x,'id.',7x,'dof',10x,'value',10x)/)
3020  format(5x,i5,5x,i5,f17.4)

      end
c
c     ==================================================================
      subroutine getelm (nel ,  ncn ,   node,  rode, idv1,  idv2 ,
	1idv3, idv4, maxel) 
c
      implicit double precision(a-h,o-z)
c
c     arguments
c     =========
c     ncn    number of nodes per element
c     node   array for element connectivity data
c     idv1   input device id. 
c     idv2   output device id. 
c     maxel  see below
c
      dimension node (maxel, ncn) 
	integer rode(maxel,27)
c
              do 6010 iel = 1 ,nel
6010  if (.not. eof(51))read (idv1,1010) iel ,(node(iel,icn),icn=1,ncn)
      print*, "nodal connectivity array read"
      write(idv2,3010)
               do 6020 jel = 1 ,nel
6020  write(idv2,3020) jel ,(node(jel,icn),icn=1,ncn)

              do 7015 iel = 1 ,nel
7015	  if (.not. eof(81))read (idv3,1020) iel ,(rode(iel,icn),icn=1,4)

			do 7017 jel = 1 ,nel
7017  write(idv4,3030) jel ,(rode(jel,icn),icn=1,4)

c			do 7021 jel = 1,nel
c			do 7023	icn = 1,4
c	print *, rode(jel,icn)
c7023	end do
c7021	end do

c
      return
c
1010  format(10i8)
1020  format(5i8)
3010  format(' ',///,' ',20('*'),' element connectivity ',20('*'),//
     1' ','id.',5x,'n o d a l - p o i n t  e n t r i e s',/)
3020  format(i3,5x,i8,5x,i8,5x,i8,5x,i8,5x,i8,5x,i8,5x,i8,5x,i8,5x,
     1i8)
3030  format(i3,5x,i8,5x,i8,5x,i8,5x,i8)
7011	format(5i8)
      end
c
c     =================================================================
      subroutine getmat  (nel ,  nmat,   pmat,  idv1,  idv2   ,  maxel,  
	1                    rtem,  rpef                                 )
c
c
      implicit double precision(a-h,o-z)
c
c     arguments
c     =========
c     nmat  number of materials
c     pmat  array for material constants for each element
c     idv1 input device id.
c     idv2  output devide id.
c     maxel see below
c
      dimension pmat (maxel,   9)
c
      write(idv2,3010)
c
      do 6010 imat = 1 ,nmat
      if (.not. eof(51)) read(idv1,1010) rvisc, power, tref, 
	1                                   pref, roden, gamad 
      
	print*, "material properties read"
                ifrom = 1
                ito   = nel
                
			  if(rtem .eq.0.) rtem  = 0.001
                if(rpef .eq.0.) rpef  = 0.001
      
	do 6020 iel  = ifrom ,ito
         pmat(iel,1) = rvisc
         pmat(iel,2) = pref
         pmat(iel,3) = power
         pmat(iel,4) = tref
         pmat(iel,5) = tbco
         pmat(iel,6) = taco
         pmat(iel,7) = dispc
	   pmat(iel,8) = roden
         pmat(iel,9) = gamad
         rtem        = tref
	   rpef        = pref 

c     roden    density
c     rvisc    mu nought; consistency coefficient
c     pref     reference presure
c     power    power law index
c     tref     reference temperature
c     tbco     coefficient b in the power law model
c     taco     coefficient a in the power law model
c     dispc    dispersion coefficient
c     gamad    shear rate
c
6020  continue
c
      write(idv2,3020) imat ,ifrom ,ito ,rvisc ,power
      write(idv2,3030)
      write(idv2,3040) tref ,tbco, pref, taco
      write(idv2,3050)
      write(idv2,3060) dispc , roden , gamad

6010  continue
      return
c

1010  format(610.5)

3010  format(' ',//' ',35('*'),' material properties ',35('*'),//
     1     ' ',2x,'id.',5x,'eid.(from-to)',3x,'consistency co-efficient'
     2,5x,'power law index',/)
3020  format(' ',i3,i12,i4,5x,g15.5,15x,g15.5)
3030  format(/x,' reference temperature   coefficient b
     1   reference presure  coefficient a '/)
3040  format(f16.3,f22.4,6x,g10.3,9x,g10.3)
3050  format(/x,
     1'dispersion coefficient     density          shear rate'/)
3060  format(g13.3,15x,g7.1,6x,g16.5)

      end
c
c================================================================
      subroutine getnod (nnp  ,qnp,cord, gord, idv1 , idv2,idv3,idv4, 
	1				   maxnp,  ndim,icord                    )
c
c
      implicit double precision(a-h,o-z)
c
c     arguments
c     =========
c     nnp    total number of nodal points in the mesh
c     cord   array for nodal coordinates
c     idv1   input device id. 
c     idv2   output device id. 
c     ndim   see below
c
      dimension cord  (maxnp, ndim), gord(maxnp,ndim)
	integer qnp
C	qnp = 5781
c
      if (.not. eof(51)) read (idv1,1010) (jnp ,(cord(jnp,idf),idf=1,2)
	1                                    ,jnp=1,nnp                  )
      if(icord.eq.0) write(idv2,3010)
      if(icord.eq.1) write(idv2,3020)
      write(idv2,3030)                    (jnp ,(cord(jnp,idf),idf=1,2)
	1                                    ,jnp=1,nnp                  )

      if (.not. eof(51)) read (idv3,1010) (jnp ,(gord(jnp,idf),idf=1,2)
	1                                    ,jnp=1,qnp                  )
      write(idv4,3030)                    (jnp ,(gord(jnp,idf),idf=1,2)
	1                                    ,jnp=1,qnp                  )


      return

1010  format(i5,e20.8,e20.8)
3010  format(' ',///' ',20('*'),' nodal coordinates ',20('*'),//
     1' ',(7x,'id.',13x,'x-coord',13x,'y-coord',20x)/)
3020  format(' ',///' ',20('*'),' nodal coordinates ',20('*'),//
     1' ',2(7x,'id/',7x,'r-coord',7x,'z-coord',20x)/)
3030  format(' ',i10,10x,f10.6,10x,f10.6)
c
      end
c
c
c
c 
c     ==================================================================
      subroutine output (nnp,qnp,soln,roln,maxdf, maxnp, icord)

c
      implicit double precision(a-h,o-z)
c
c     arguments are already defined
c     =============================
      dimension soln(maxdf), roln(maxdf), vm(maxnp), vm4(maxnp)
	integer qnp

	
c	qnp = 186

	write(60,3010)

      if(icord.eq.0) write(60,3020)
      if(icord.eq.1) write(60,3030)
	
	write(101,3010)

      if(icord.eq.0) write(101,3020)
      if(icord.eq.1) write(101,3030)

      do 6010 inp = 1,nnp
                        jnp = inp + nnp
                        mnp = jnp + nnp
					  vm(inp) = sqrt(soln(inp)**2 + soln(jnp)**2)


      write(60,3040)inp ,soln(inp),soln(jnp),soln(mnp)
	      write(*,3040)inp ,soln(inp),soln(jnp),vm(inp),soln(mnp)


6010  continue

	do 6020 inp = 1,qnp
					  jnp = inp + qnp
					  mnp = jnp + qnp
					  vm4(inp) = sqrt(roln(inp)**2 + roln(jnp)**2)
	write(101,3050)inp ,roln(inp), roln(jnp), vm4(maxnp), roln(mnp)

6020	continue

3010  format(/' nodal velocities and  presures '/)
3020  format('  id.      ux         uy              pres' /)
3030  format('  id.      ur         uz              pres' /)
3040  format(i5,3e13.4,e22.8)
3050  format(i5,3e13.4,e22.8)



      return
      end
c
c
c    ==================================================================
      subroutine putbcv
     1    (nnp  ,   nbc  ,    ibc  ,   jbc  ,   vbc  ,  ncod,    bc   ,
     2     maxbc,   maxdf,    maxel,   node ,   nel    				)
c
      implicit double precision(a-h,o-z)
c
c     arguments
c     =========
c     ncod    array for constraint switch defined for every d.o.f.
c     bc      array for storing contraint value
c     maxbc   see below
c     maxdf   see below
c
      dimension ibc  (maxbc) ,jbc  (maxbc) ,vbc  (maxbc   )
      dimension ncod (maxdf) ,bc   (maxdf) ,node (maxel,27)
c
      do 6010 ind = 1 ,nbc
      if(jbc(ind)>2) goto 6010                 
      jnd         = ibc(ind)+(jbc(ind)-1)*nnp
      bc    (jnd) = vbc(ind)
      ncod  (jnd) =  1
 6010 continue
	
	ia=nnp*2+1
	ib=nnp*3
	do 5001 ic=ia,ib
	ncod(ic)=1
	bc  (ic)=0.0
 5001 continue

c *** PICK UP THE CORNER NODES

  	do 5002 iel = 1 ,nel
	do 5003 inp =19 , 25, 2
	kc=iabs(node(iel,inp))
	ncod(kc)=0
 5003 continue 
 5002 continue

c      ncod(3822)=1.0
c      bc  (3822)=0.0
 

	return
      end
c
c     =============================================================== 
      subroutine secinv
     1   (nel  ,nnp  ,ncn  ,ngaus,node ,sinv ,cord ,p  ,b,
     2    del  ,da   ,dis  ,maxnp,maxel,maxst,ndim ,icord,
     3    maxdf,num)

c
      implicit double precision(a-h,o-z)
c
c     function
c     --------
c     
c     calculates the second invariant of rate of deformation
c     tensor at integration points.
c
c
      dimension vel  (maxdf      ) ,cord (maxnp, ndim)
      dimension node (maxel,maxst) ,sinv (maxel,  ncn)
      dimension p    (    9      ) ,del  (    2,    9)
      dimension b    (    2,    9)
c
                                         rewind 15
                                     
      do 5000 iel= 1 , nel
              lg = 0
      do 5010 ig = 1 ,ngaus
      do 5010 jg = 1 ,ngaus
              lg = lg+1
c
           if(.not. eof(15)) read (15) iiel,iig,jjg,p,del,b,da
c
                  x1  = 0.0
                  u1  = 0.0
                  u11 = 0.0
                  u12 = 0.0
                  u21 = 0.0
                  u22 = 0.0

          do 5020 icn = 1 ,ncn
                                    jcn = iabs(node(iel,icn))
                                    mcn = jcn + nnp
c
c     components of the rate of deformation tensor
c
                  u11 = u11 + b(1,icn)*vel(jcn)
                  u12 = u12 + b(2,icn)*vel(jcn)
                  u21 = u21 + b(1,icn)*vel(mcn)
                  u22 = u22 + b(2,icn)*vel(mcn)
 5020     continue
c
c     second invariant of the rate of deformation tensor
c
      sinv(iel,lg)=0.125*((u11+u11)*(u11+u11)+
     1                    (u12+u21)*(u12+u21)+
     2                    (u21+u12)*(u21+u12)+
     3                    (u22+u22)*(u22+u22))
     
 5010 continue
 5000 continue 

      return
      end
c
c
c    ==================================================================
      subroutine setprm
     1    (nnp  ,   nel   ,   ncn  ,   node ,   ndf  ,  maxel,   maxst,
     2     ndn  ,   ntrix ,   maxdf,   mdf  ,  nopp          )
c
      implicit double precision(a-h,o-z)
c
c     arguments
c     =========
c     all arguments are defined elsewhere.
c
      dimension node (maxel,maxst), ndn  (maxdf)
      dimension mdf  (maxdf      ), nopp (maxdf)
c
c     function
c     ========
c     Sets the location data for nodal degrees of freedom
c
      ntotal = ndf * nnp
	ntrix  = ndf * ncn

      do 6010 iel = 1 ,nel
         ndn(iel) = ntrix
      do 6010 icn = 1 ,ncn
              kcn =node(iel,icn)
              jcn =icn+ncn
              lcn =kcn+nnp
      
	node(iel,jcn) = lcn
	jmm=icn+ncn*2
	nnn=kcn+nnp*2
	node(iel,jmm) = nnn

6010  continue

      do 6020 idf  = 1,ntotal
         mdf(idf ) = 1
         nopp(idf) = idf
6020  continue
c
      return
      end
c
c     ==========================================================
      subroutine shape ( xi    ,   eta ,    p ,    del ,    ncn)
c
      implicit double precision (a-h,o-z)
c
      dimension p(9) ,del(2,9)
 
      if (ncn.eq.4) then
      
	    del(1,1)=-0.25*(1-eta)
          del(1,2)= 0.25*(1-eta)
          del(1,3)= 0.25*(1+eta)
          del(1,4)=-0.25*(1+eta)

          del(2,1)=-0.25*(1-xi)
          del(2,2)=-0.25*(1+xi)
          del(2,3)= 0.25*(1+xi)
          del(2,4)= 0.25*(1-xi)

          p(1)=0.25*(1-xi)*(1-eta)
          p(2)=0.25*(1+xi)*(1-eta)
          p(3)=0.25*(1+xi)*(1+eta)
          p(4)=0.25*(1-xi)*(1+eta)

      elseif (ncn.eq.8) then

          del(1,1)=0.5*xi-0.5*xi*eta-0.25*eta**2+0.25*eta
          del(1,2)=0.5*xi-0.5*xi*eta+0.25*eta**2-0.25*eta
          del(1,3)=0.5*xi+0.5*xi*eta+0.25*eta**2+0.25*eta
          del(1,4)=0.5*xi+0.5*xi*eta-0.25*eta**2-0.25*eta
          del(1,5)=xi*(-1+eta)
          del(1,6)=0.5-0.5*eta**2
          del(1,7)=-xi*(1+eta)
          del(1,8)=-0.5+0.5*eta**2

          del(2,1)=0.5*eta-0.25*xi**2-0.5*xi*eta+0.25*xi
          del(2,2)=0.5*eta-0.25*xi**2+0.5*eta*xi-0.25*xi
          del(2,3)=0.5*eta+0.25*xi**2+0.5*xi*eta+0.25*xi
          del(2,4)=0.5*eta+0.25*xi**2-0.5*xi*eta-0.25*xi
          del(2,5)=-0.5+0.5*xi**2
          del(2,6)=-(1+xi)*eta
          del(2,7)=0.5-0.5*xi**2
          del(2,8)=(-1+xi)*eta

          p(1)=0.25*(1-xi)*(1-eta)*(-1-xi*eta)
          p(2)=0.25*(1+xi)*(1-eta)*(-1+xi-eta)
          p(3)=0.25*(1+xi)*(1+eta)*(-1+xi+eta)
          p(4)=0.25*(1-xi)*(1+eta)*(-1-xi+eta)
          p(5)=0.5*(1-xi**2)*(1-eta)
          p(6)=0.5*(1+xi)*(1-eta**2)
          p(7)=0.5*(1-xi**2)*(1+eta)
          p(8)=0.5*(1-xi)*(1-eta**2)

      elseif (ncn.eq.9) then

          del(1,1)=0.25*(2*xi-1)*(eta**2-eta)
          del(1,2)= -xi*(eta**2-eta)
          del(1,3)=0.25*(2*xi+1)*(eta**2-eta)
          del(1,4)=0.5 *(2*xi+1)*(1-eta**2)
          del(1,5)=0.25*(2*xi+1)*(eta**2+eta)
          del(1,6)=-xi *(eta**2+eta)
          del(1,7)=0.25*(2*xi-1)*(eta**2+eta)
          del(1,8)=0.5 *(2*xi-1)*(1-eta**2)
          del(1,9)=-2  *xi*(1-eta**2)

          del(2,1)=0.25*(xi**2-xi)*(2*eta-1)
          del(2,2)=0.5 *(1-xi**2)*(2*eta-1)
          del(2,3)=0.25*(xi**2+xi)*(2*eta-1)
          del(2,4)=-eta*(xi**2+xi)
          del(2,5)=0.25*(xi**2+xi)*(2*eta+1)
          del(2,6)=0.5 *(1-xi**2)*(2*eta+1)
          del(2,7)=0.25*(xi**2-xi)*(2*eta+1)
          del(2,8)=-eta*(xi**2-xi)
          del(2,9)=-2  *eta*(1-xi**2)

          p(1)=0.25*(xi**2-xi)*(eta**2-eta)
          p(2)=0.5*(1-xi**2)*(eta**2-eta)
          p(3)=0.25*(xi**2+xi)*(eta**2-eta)
          p(4)=0.5*(xi**2+xi)*(1-eta**2)
          p(5)=0.25*(xi**2+xi)*(eta**2+eta)
          p(6)=0.5*(1-xi**2)*(eta**2+eta)
          p(7)=0.25*(xi**2-xi)*(eta**2+eta)
          p(8)=0.5*(xi**2-xi)*(1-eta**2)
          p(9)=(1-xi**2)*(1-eta**2)

      endif
      return
      end
c
c
c     ==================================================================
      subroutine visca
	1                (rvisc   ,power   ,visc   ,stemp   ,rtem   ,tbco,
     2                 spres  ,rpef    ,taco   ,gamad                ) 
c
      implicit double precision(a-h,o-z)
c       

      visc   = rvisc*((4*gamad)**((power-1.0)/2))

      return
      end
c


c     ==================================================================

	subroutine tecplot
	
	1(nnp, nel, qnp, qel, maxel, maxst, maxnp, ndim,
	2 vel, pres, rel, qres, cord, node, gord, mode) 


      implicit double precision(a-h,o-z)



	dimension x    (42000), y    (42000)
	dimension vx   (42000), vy   (42000)
	dimension vm   (42000), p	 (42000)

	dimension x4   (42000), y4   (42000)
	dimension vx4  (42000), vy4  (42000)
	dimension vm4  (42000), p4	 (42000)


	dimension vel (maxnp, ndim), pres(maxnp      )
	dimension rel (maxnp, ndim), qres(maxnp      )

	dimension cord(maxnp, ndim), node(maxel,maxst)
	dimension gord(maxnp, ndim), mode(maxel,maxst)

	dimension na   (42000), nb   (42000), nc   (42000)
	dimension nd   (42000), ne   (42000), nf   (42000)
	dimension ng   (42000), nh   (42000), ni   (42000)

	dimension na4  (42000), nb4  (42000)
	dimension nc4  (42000), nd4  (42000)



	integer qnp, qel


	jel = 4*nel

	do 8001 inp = 1, nnp

		vx(inp) = vel(inp,1)
		vy(inp) = vel(inp,2)
		p (inp) = pres(inp)
		x (inp) = cord(inp, 1)
		y (inp) = cord(inp, 2)
		vm(inp) = sqrt(vx(inp)**2 + vy(inp)**2)

8001	continue


	do 8003 j = 1, nel

		na(j) = iabs(node(j,1))
		nb(j) = iabs(node(j,2))
		nc(j) = iabs(node(j,3))
		nd(j) = iabs(node(j,4))
		ne(j) = iabs(node(j,5))
		nf(j) = iabs(node(j,6))
		ng(j) = iabs(node(j,7))
		nh(j) = iabs(node(j,8))
		ni(j) = iabs(node(j,9))

8003	continue


	write(301,1000)
	write(301,2000) nnp, jel


	do 8005 i = 1, nnp

		write (301,3000) x(i), y(i), vx(i), vy(i), vm(i), p(i)
		
8005	continue



	do 8007 j = 1, nel

		write (301,4000) na(j), nb(j), ni(j), nh(j)
		write (301,4000) nb(j), nc(j), nd(j), ni(j)
		write (301,4000) ni(j), nd(j), ne(j), nf(j)
		write (301,4000) nh(j), ni(j), nf(j), ng(j)

8007	continue			 


c	==========================================================================

	print *, "qnp = ", qnp

	print *, "qel = ", qel


	do 8009 inp = 1, qnp

		vx4(inp) = rel(inp,  1)
		vy4(inp) = rel(inp,  2)
		p4 (inp) = qres(inp   )
		x4 (inp) = gord(inp, 1)
		y4 (inp) = gord(inp, 2)
		vm4(inp) = sqrt(vx4(inp)**2 + vy4(inp)**2)

8009	continue


	
	do 8011 j = 1, qel

		na4(j) = iabs(mode(j,1))
		nb4(j) = iabs(mode(j,2))
		nc4(j) = iabs(mode(j,3))
		nd4(j) = iabs(mode(j,4))

8011	continue		


	write(401,1000)
	write(401,2000) qnp, qel


      do 8013 i = 1, qnp
	
		write (401,3000) x4(j), y4(j), vx4(j), vy4(j), vm4(j), p4(j)
      
8013	continue 



	do 8015 j = 1, qel

		write (401,4000) na4(j), nb4(j), nc4(j), nd4(j)
      
8015	continue 			 


c	==========================================================================


1000	format ('Variables = "X", "Y", "U", "V", "M", "P"')
2000	format ('ZONE',1x,'N=',i5,1x,',','E=',i5,1x,',','F=FEPOINT',',',
	1        'ET=QUADRILATERAL')
3000  format (e20.12,e20.12,e20.12,e20.12,e20.12,e22.8) 
4000  format (4i8)   


	return

	end


c     ============================================================
c     E    N   D          O   F         P   R   O   G   R   A   M   
c     ============================================================

